#!/usr/bin/env node
"use strict";
exports.__esModule = true;
var packageJson = require('./package.json');
var process_1 = require("process");
var ts_morph_1 = require("ts-morph");
var src_1 = require("./packages/ast/src");
console.log("[" + packageJson.name + " v." + packageJson.version + "]");
if (process.argv.length < 3) {
    console.log("You have to provide path to tsconfig file");
    process_1.exit();
}
var files = src_1.getProjectFiles(process.argv[2]);
var componentFiles = src_1.getComponentFiles(files);
var selectorUsages = getComponentUsagesBySelector(componentFiles);
var classNameUsages = getComponentUsagesByClassName(componentFiles);
var usages = componentFiles.map(function (_a) {
    var file = _a.file;
    return ({
        filePath: file.getFilePath(),
        selectorUsage: selectorUsages[file.getFilePath()],
        classUsage: classNameUsages[file.getFilePath()],
        probablyUnused: selectorUsages[file.getFilePath()] + classNameUsages[file.getFilePath()] <=
            0
    });
});
function getComponentUsagesBySelector(componentFiles) {
    return componentFiles.reduce(function (result, file, index, allFiles) {
        result[file.file.getFilePath()] = allFiles.filter(function (f) {
            return f.componentTemplateSource.includes(file.componentSelector);
        }).length;
        return result;
    }, {});
}
function getComponentUsagesByClassName(componentFiles) {
    return componentFiles.reduce(function (result, file, index, allFiles) {
        result[file.file.getFilePath()] = src_1.getClassReferencingNodesInOtherFiles(file.file, file.className, /\.spec\.ts$/)
            .filter(function (node) {
            return !ts_morph_1.Node.isImportSpecifier(node.getParent());
        })
            .filter(function (node) { return !src_1.isNgModuleField(node, src_1.NgModuleMetadataField.declarations); })
            .filter(function (node) { return !src_1.isNgModuleField(node, src_1.NgModuleMetadataField.exports); }).length;
        return result;
    }, {});
}
console.log('Probably unused components:');
console.log(usages
    .filter(function (_a) {
    var probablyUnused = _a.probablyUnused;
    return probablyUnused;
})
    .map(function (_a) {
    var filePath = _a.filePath;
    return filePath;
}).join("\n"));
