"use strict";
exports.__esModule = true;
exports.getAncestorArrayName = exports.isNgModuleField = exports.getClassReferencingNodesInOtherFiles = exports.getPropertyValueFromDecorator = exports.getClassNameWithDecorator = exports.getDecorator = exports.hasDecorator = exports.getComponentFiles = exports.getProjectFiles = exports.NgModuleMetadataField = void 0;
var fs_1 = require("fs");
var path_1 = require("path");
var ts_morph_1 = require("ts-morph");
var NgModuleMetadataField;
(function (NgModuleMetadataField) {
    NgModuleMetadataField["declarations"] = "declarations";
    NgModuleMetadataField["exports"] = "exports";
    NgModuleMetadataField["entryComponents"] = "entryComponents";
    NgModuleMetadataField["imports"] = "imports";
})(NgModuleMetadataField = exports.NgModuleMetadataField || (exports.NgModuleMetadataField = {}));
function getProjectFiles(tsConfigFilePath) {
    return new ts_morph_1.Project({
        tsConfigFilePath: tsConfigFilePath
    }).getSourceFiles();
}
exports.getProjectFiles = getProjectFiles;
function getComponentFiles(sourceFiles) {
    return sourceFiles.filter(hasDecorator("Component")).map(function (file) {
        var componentTemplateSource = "";
        var componentDecorator = getDecorator("Component")(file);
        var inlineTemplate = exports.getPropertyValueFromDecorator(componentDecorator, "template");
        var templateUrl = exports.getPropertyValueFromDecorator(componentDecorator, "templateUrl");
        if (!inlineTemplate && templateUrl) {
            componentTemplateSource = fs_1.readFileSync(path_1.resolve(file.getDirectoryPath(), templateUrl), { encoding: "utf-8" });
        }
        else {
            componentTemplateSource = inlineTemplate || "";
        }
        return {
            file: file,
            componentSelector: exports.getPropertyValueFromDecorator(componentDecorator, "selector") || "",
            componentTemplateSource: componentTemplateSource,
            className: getClassNameWithDecorator("Component")(file)
        };
    });
}
exports.getComponentFiles = getComponentFiles;
// source file utils
function hasDecorator(decoratorName) {
    return function (file) {
        return file
            .getClasses()
            .some(function (classDeclaration) { return classDeclaration.getDecorator(decoratorName); });
    };
}
exports.hasDecorator = hasDecorator;
function getDecorator(decoratorName) {
    return function (file) {
        return file
            .getClasses()
            .filter(function (classDeclaration) {
            return classDeclaration.getDecorator(decoratorName);
        })[0]
            .getDecorator(decoratorName);
    };
}
exports.getDecorator = getDecorator;
function getClassNameWithDecorator(decoratorName) {
    return function (file) {
        return file
            .getClasses()
            .filter(function (classDeclaration) {
            return classDeclaration.getDecorator(decoratorName);
        })[0]
            .getName() || "";
    };
}
exports.getClassNameWithDecorator = getClassNameWithDecorator;
// decorator utils
var getPropertyValueFromDecorator = function (componentDecorator, property) {
    var value = null;
    componentDecorator === null || componentDecorator === void 0 ? void 0 : componentDecorator.getArguments().forEach(function (argument) {
        var argumentValueIfSelector = argument
            .getDescendantsOfKind(ts_morph_1.SyntaxKind.PropertyAssignment)
            .filter(function (child) { return (child === null || child === void 0 ? void 0 : child.getName()) === property; })
            .map(function (child) { var _a; return (_a = child.getInitializer()) === null || _a === void 0 ? void 0 : _a.getText(); });
        if (argumentValueIfSelector === null || argumentValueIfSelector === void 0 ? void 0 : argumentValueIfSelector.length) {
            value = argumentValueIfSelector[0] || null;
            return;
        }
    });
    return typeof value === "string"
        ? value.replace(/['"]/g, "")
        : value;
};
exports.getPropertyValueFromDecorator = getPropertyValueFromDecorator;
var getClassReferencingNodesInOtherFiles = function (file, className, excludedFileNamePattern) {
    var _a;
    return ((_a = file
        .getDescendantsOfKind(ts_morph_1.SyntaxKind.ClassDeclaration)
        .filter(function (node) { var _a; return ((_a = node === null || node === void 0 ? void 0 : node.getSymbol()) === null || _a === void 0 ? void 0 : _a.getEscapedName()) === className; })[0]) === null || _a === void 0 ? void 0 : _a.findReferencesAsNodes().filter(function (node) {
        var sourceFile = node.getSourceFile();
        return (sourceFile.getFilePath() !== file.getFilePath() &&
            (!excludedFileNamePattern ||
                !excludedFileNamePattern.test(sourceFile.getFilePath())));
    }).reduce(function (references, referencingNode) {
        references.push(referencingNode);
        return references;
    }, [])) || [];
};
exports.getClassReferencingNodesInOtherFiles = getClassReferencingNodesInOtherFiles;
var isNgModuleField = function (node, field) { return exports.getAncestorArrayName(node) === field; };
exports.isNgModuleField = isNgModuleField;
var getAncestorArrayName = function (node) {
    var _a, _b, _c;
    return (_c = (_b = (_a = node
        .getParentIfKind(ts_morph_1.SyntaxKind.ArrayLiteralExpression)) === null || _a === void 0 ? void 0 : _a.getParentIfKind(ts_morph_1.SyntaxKind.PropertyAssignment)) === null || _b === void 0 ? void 0 : _b.getSymbol()) === null || _c === void 0 ? void 0 : _c.getEscapedName();
};
exports.getAncestorArrayName = getAncestorArrayName;
